EXPORTS
	??0CAsyncRequest@@QAE@PAVCScsSession@@PAVCScsSubsession@@ABVRMessage2@@@Z @ 1 NONAME ; CAsyncRequest::CAsyncRequest(class CScsSession *, class CScsSubsession *, class RMessage2 const &)
	??0CScsServer@@IAE@ABVTVersion@@W4TPriority@CActive@@@Z @ 2 NONAME ; CScsServer::CScsServer(class TVersion const &, enum CActive::TPriority)
	??0CScsSession@@QAE@AAVCScsServer@@@Z @ 3 NONAME ; CScsSession::CScsSession(class CScsServer &)
	??0CScsSubsession@@IAE@AAVCScsSession@@@Z @ 4 NONAME ; CScsSubsession::CScsSubsession(class CScsSession &)
	??1CScsServer@@UAE@XZ @ 5 NONAME ; CScsServer::~CScsServer(void)
	??1CScsSession@@MAE@XZ @ 6 NONAME ; CScsSession::~CScsSession(void)
	?CompleteAndMarkForDeletion@CAsyncRequest@@IAEXH@Z @ 7 NONAME ; void CAsyncRequest::CompleteAndMarkForDeletion(int)
	?ConstructL@CScsServer@@IAEXH@Z @ 8 NONAME ; void CScsServer::ConstructL(int)
	?ConstructL@CScsSession@@IAEXXZ @ 9 NONAME ; void CScsSession::ConstructL(void)
	?DoCreateSubsessionL@CScsSession@@EAEPAVCScsSubsession@@HABVRMessage2@@@Z @ 10 NONAME ; class CScsSubsession * CScsSession::DoCreateSubsessionL(int, class RMessage2 const &)
	?TransferToScsFrameworkL@CAsyncRequest@@QAEXXZ @ 11 NONAME ; void CAsyncRequest::TransferToScsFrameworkL(void)
	?NewSessionL@CScsServer@@MBEPAVCSession2@@ABVTVersion@@ABVRMessage2@@@Z @ 12 NONAME ; class CSession2 * CScsServer::NewSessionL(class TVersion const &, class RMessage2 const &) const
	?RunError@CAsyncRequest@@MAEHH@Z @ 13 NONAME ; int CAsyncRequest::RunError(int)
	?RunL@CAsyncRequest@@MAEXXZ @ 14 NONAME ; void CAsyncRequest::RunL(void)
	?ServiceError@CScsSession@@MAEXABVRMessage2@@H@Z @ 15 NONAME ; void CScsSession::ServiceError(class RMessage2 const &, int)
	?ServiceL@CScsSession@@MAEXABVRMessage2@@@Z @ 16 NONAME ; void CScsSession::ServiceL(class RMessage2 const &)
	?ShutdownWhenIdleL@CScsServer@@QAEXXZ @ 17 NONAME ; void CScsServer::ShutdownWhenIdleL(void)
	?StartScsServer@@YAHP6APAVCScsServer@@XZ@Z @ 18 NONAME ; int StartScsServer(class CScsServer * (*)(void))
	??0CScsServer@@IAE@ABVTVersion@@ABVTPolicy@CPolicyServer@@W4TPriority@CActive@@@Z @ 19 NONAME ; CScsServer::CScsServer(class TVersion const &, class CPolicyServer::TPolicy const &, enum CActive::TPriority)
	?DoCleanup@CAsyncRequest@@UAEXXZ @ 20 NONAME ; void CAsyncRequest::DoCleanup(void)
	?DoPostHeapMarkOrCheckL@CScsServer@@UAEXXZ @ 21 NONAME ; void CScsServer::DoPostHeapMarkOrCheckL(void)
	?DoPreHeapMarkOrCheckL@CScsServer@@UAEXXZ @ 22 NONAME ; void CScsServer::DoPreHeapMarkOrCheckL(void)
	?DisableShutdownTimer@CScsServer@@IAEXXZ @ 23 NONAME ; void CScsServer::DisableShutdownTimer(void)
	?EnableShutdownTimerL@CScsServer@@IAEXH@Z @ 24 NONAME ; void CScsServer::EnableShutdownTimerL(int)
	?CloseAllSubsessionsL@CScsSession@@IAEXXZ @ 25 NONAME ; void CScsSession::CloseAllSubsessionsL(void)

